@@ -15,6 +15,9 @@ logger = logging.getLogger(__name__)
 
 _visited_objects = []
 
+# A list of function docstring pre-processing hooks
+function_docstring_preprocessing_hooks: List[Callable[[str], str]] = []
+
 
 class DirectoryWalkerGuard(object):
 
@@ -33,7 +36,7 @@ class DirectoryWalkerGuard(object):
         os.chdir(os.path.pardir)
 
 
-_default_pybind11_repr_re = re.compile(r'<\w+(\.\w+)* object at 0x[0-9a-fA-F]+>')
+_default_pybind11_repr_re = re.compile(r'(<\w+(\.\w+)* object at 0x[0-9a-fA-F]+>)|(<\w+(.\w+)*: \d+>)')
 
 
 def replace_default_pybind11_repr_with_ellipses(line):
@@ -239,9 +242,13 @@ class StubsGenerator(object):
                               r"\s*->\s*" \
                               r"(?P<rtype>[^\(\)]+)\s*".format(name=name,
                                                                balanced_parentheses=parentheses_three_fold)
-            doc_lines = func.__doc__
+            docstring = func.__doc__
+
+            for hook in function_docstring_preprocessing_hooks:
+                docstring = hook(docstring)
+
             signatures = []
-            for line in doc_lines.split("\n"):
+            for line in docstring.split("\n"):
                 m = re.match(signature_regex, line)
                 if m:
                     args = m.group("args")
@@ -279,7 +286,7 @@ class StubsGenerator(object):
                 for line in prop.fget.__doc__.split("\n"):
                     if strip_module_name:
                         line = line.replace(module_name + ".", "")
-                    m = re.match(r"\s*\((?P<args>[^()]*)\)\s*->\s*(?P<rtype>[^()]+)\s*", line)
+                    m = re.match(r"\s*(\w*)\((?P<args>[^()]*)\)\s*->\s*(?P<rtype>[^()]+)\s*", line)
                     if m:
                         getter_rtype = m.group("rtype")
                         break
@@ -290,7 +297,7 @@ class StubsGenerator(object):
                 for line in prop.fset.__doc__.split("\n"):
                     if strip_module_name:
                         line = line.replace(module_name + ".", "")
-                    m = re.match(r"\s*\((?P<args>[^()]*)\)\s*->\s*(?P<rtype>[^()]+)\s*", line)
+                    m = re.match(r"\s*(\w*)\((?P<args>[^()]*)\)\s*->\s*(?P<rtype>[^()]+)\s*", line)
                     if m:
                         args = m.group("args")
                         # replace first argument with self
@@ -306,8 +313,11 @@ class StubsGenerator(object):
         if docstring is None:
             return ""
 
+        for hook in function_docstring_preprocessing_hooks:
+            docstring = hook(docstring)
+
         signature_regex = r"(\s*(?P<overload_number>\d+).\s*)" \
-                          r"?{name}\s*\((?P<args>[^\(\)]*)\)\s*(->\s*(?P<rtype>[^\(\)]+)\s*)?".format(name=r"\w+")
+                          r"?{name}\s*\((?P<args>.*)\)\s*(->\s*(?P<rtype>[^\(\)]+)\s*)?".format(name=r"\w+")
 
         lines = docstring.split("\n\n")
         lines = filter(lambda line: line != "Overloaded function.", lines)
@@ -572,6 +582,8 @@ class ClassStubsGenerator(StubsGenerator):
             # check if attribute is in __dict__ (fast path) before slower search in base classes
             if name not in self.klass.__dict__ and is_base_member(name, member):
                 continue
+            if name.startswith('__pybind11_module'):
+                continue
             if inspect.isroutine(member):
                 self.methods.append(ClassMemberStubsGenerator(name, member, self.klass.__module__))
             elif name != '__class__' and inspect.isclass(member):
@@ -837,7 +849,7 @@ def recursive_mkdir_walker(subdirs, callback):  # type: (List[str], Callable) ->
             recursive_mkdir_walker(subdirs[1:], callback)
 
 
-def main():
+def main(args=None):
     parser = ArgumentParser(prog='pybind11-stubgen', description="Generates stubs for specified modules")
     parser.add_argument("-o", "--output-dir", help="the root directory for output stubs", default="./stubs")
     parser.add_argument("--root-module-suffix", type=str, default="-stubs", dest='root_module_suffix',
@@ -855,7 +867,7 @@ def main():
     parser.add_argument("module_names", nargs="+", metavar="MODULE_NAME", type=str, help="modules names")
     parser.add_argument("--log-level", default="INFO", help="Set output log level")
 
-    sys_args = parser.parse_args()
+    sys_args = parser.parse_args(args or sys.argv[1:])
 
     if sys_args.non_stop:
         sys_args.ignore_invalid = ['all']
